﻿package
{
	import flash.display.*;
	import flash.events.*;
	
	import org.papervision3d.cameras.*;
	import org.papervision3d.view.*;
	import org.papervision3d.materials.*;
	import org.papervision3d.objects.*;
	import org.papervision3d.objects.primitives.*
	import org.papervision3d.materials.utils.*;
	
	import caurina.transitions.Tweener;
	
	/**
	* フォトプリントエフェクト
	*/
	public class Main extends BasicView 
	{	
		// 写真のパーティクル分割数
		static private const MAX_CELL_W:uint = 20;
		static private const MAX_CELL_H:uint = 15;

		// 写真のパーティクル保持配列(二次元配列)
		private var particlePlaneArr:Array /* type of Array */ = [];
		
		// パーティクルのコンテナ
		private var wrap:DisplayObject3D;
		
		// 放射線ブラー	
		private var radiationBlurPlane:Plane;
		private var imgSprite:Sprite = new BitmapImage();
		
		// 放射線ブラーの適用に使う変数
		private var renderBmpData:BitmapData;
		private var radiationBlurMaterial:BitmapMaterial;
		
		// 背景レイヤー
		private var base:Plane;
		
		/**
		 * コンストラクタ
		 */
		public function Main()
		{
			// BasicViewの初期化
			super(0, 0, true, false, CameraType.TARGET);
			
			// init swf
			stage.quality = StageQuality.MEDIUM;
			
			init();
			
			// レンダリングを開始します
			startRendering();
			
			// ボタンのイベント
			btn.addEventListener(MouseEvent.CLICK, function(event:Event):void
			{
				startMotion();
			});
		}
		
		/**
		 * 初期化
		 */
		private function init():void
		{
			//--------------------------------
			// カメラ設定
			//--------------------------------
			camera.focus = 200;
			camera.zoom = 1.5;
			
			
			//--------------------------------
			// フォトパーティクルの作成
			//--------------------------------
			var folder:Sprite = new Sprite();
			imgSprite = new BitmapImage();
			folder.addChild(imgSprite);
			
			// パーティクルの保存配列
			var bmpDataArr:Array /* type of BitmapData */ = [];
			
			var i:int = 0;
			var j:int = 0;
			
			// パーティクルを分割して保存するループ
			for (i = 0; i < MAX_CELL_W; i++ )
			{
				for (j = 0; j < MAX_CELL_H; j++ )
				{
					imgSprite.x = - imgSprite.width / MAX_CELL_W * i;
					imgSprite.y = - imgSprite.height / MAX_CELL_H * j;
					
					var bmpData:BitmapData = new BitmapData(
						imgSprite.width / MAX_CELL_W, 
						imgSprite.height/ MAX_CELL_H, 
						false, 
						0xFFFF0000
					);
					
					bmpData.draw(folder);
					bmpDataArr.push(bmpData);
				}
			}
			
			// パーティクルを追加する親オブジェクトを作成
			wrap = new DisplayObject3D();
			scene.addChild(wrap);
			wrap.visible = false;
			
			// 画面中央に設置されるように画像の半分サイズだけ寄せる
			wrap.x = - imgSprite.width / 2;
			wrap.y = - imgSprite.height / 2;
			
			// 保存されたパーティクル配列から3DのPlaneオブジェクトを作成
			for (i = 0; i < MAX_CELL_W; i++ )
			{
				// 二次元配列の作成
				particlePlaneArr[i] /* type of Plane */ = [];
				
				for (j = MAX_CELL_H; j > 0 ; j-- )
				{
					// テクスチャは上記で作成したBitmapData配列を利用
					var material:BitmapMaterial = new BitmapMaterial(BitmapData(bmpDataArr.shift()), false);
					
					// Planeの作成
					var plane:Plane = new Plane(
						material, 
						imgSprite.width / MAX_CELL_W, 
						imgSprite.height / MAX_CELL_H, 
						1, 1
					);
					
					// Planeの座標を設定
					plane.x = imgSprite.width / MAX_CELL_W * i + (imgSprite.width / MAX_CELL_W / 2);
					plane.y = imgSprite.height / MAX_CELL_H * j - (imgSprite.height / MAX_CELL_H / 2);
					
					
					// 保存と表示リストに追加
					wrap.addChild(plane);
					particlePlaneArr[i][j] = plane;
				}
			}
			
			
			//--------------------------------
			// 放射ブラーの作成
			//--------------------------------
			radiationBlurPlane = new Plane(null, imgSprite.height, imgSprite.height, 4, 4);
			// エフェクトが適用できるようにuseOwnContainerをtrueにする
			radiationBlurPlane.useOwnContainer = true;
			// 描画モードを「ADD(追加)」にする
			radiationBlurPlane.blendMode = BlendMode.ADD;
			
			// そのままだと画像分ずれているのでセンタリングする
			radiationBlurPlane.z = - imgSprite.height / 2 + 25;
			
			// そのままだと横向きなので立て向きに回転させる
			radiationBlurPlane.localRotationY = 90;
			radiationBlurPlane.localRotationZ = 90;
			
			// 3D空間に表示
			scene.addChild(radiationBlurPlane);
			radiationBlurPlane.visible = false;
			
			// 放射ブラーをキャプチャする用のBitmapDataを作成しておく
			renderBmpData = new BitmapData(200, 200, true, 0x00000000);
			
			
			//--------------------------------
			// パーティクルのゴールを設定
			//--------------------------------
			base = new Plane(
				new MovieAssetMaterial("BasePlane",true),
				imgSprite.width, imgSprite.height, 1, 1
			);
			base.z = 25;
			base.visible = false;
			scene.addChild(base);
		}
		
		
			
		/**
		* モーション開始
		*/
		private function startMotion():void
		{
			// 画面表示
			wrap.visible = true;
			radiationBlurPlane.visible = true;
			base.visible = true;
			
			var i:int = 0;
			var j:int = 0;
			
			// パーティクルフォト
			for (i = 0; i < particlePlaneArr.length; i++ )
			{
				for (j = 0; j < particlePlaneArr[i].length; j++ )
				{
					if(particlePlaneArr[i][j] == null ) continue;
					particlePlaneArr[i][j].z = -900;
					Tweener.addTween(Plane(particlePlaneArr[i][j]),
					{
						z     : 0,
						time  : 2.0,
						transition : "easeOutCubic",
						delay : i * .125 + Math.random() * 0.2
					});
				}
			}
			
			
			// 放射線ブラー
			addEventListener(Event.ENTER_FRAME, enterFrameHandler);
			
			radiationBlurPlane.x = - imgSprite.width / 2;
			radiationBlurPlane.alpha = 0;
			
			Tweener.addTween(radiationBlurPlane,
			{
				x     : imgSprite.width / 2,
				time  : 2.5,
				delay : 2,
				transition : "easeOutQuad",
				onComplete : function():void
				{
					removeEventListener(Event.ENTER_FRAME, enterFrameHandler);
				}
			});
			
			Tweener.addTween(radiationBlurPlane,
			{
				alpha : 1,
				time  : 1,
				delay : .5
			});
			Tweener.addTween(radiationBlurPlane,
			{
				alpha : 0,
				time  : 1.25,
				delay : 3.0
			});
			
			// カメラ
			
			camera.x = -600;
			camera.y = -300;
			camera.z = -800;
			
			Tweener.addTween(camera,
			{
				z     : -300,
				time  : 5,
				transition : "easeOutCubic"
			});
			
			Tweener.addTween(camera,
			{
				x     : 0,
				y     : 0,
				time  : 5,
				delay : 0.5,
				transition : "easeInOutExpo"
			});
			
			
		}
		
		private function enterFrameHandler(event:Event):void
		{
			radiationBlur.updateNoise();
			renderBmpData.draw(radiationBlur);
			
			radiationBlurMaterial = new BitmapMaterial(renderBmpData, true);
			radiationBlurMaterial.doubleSided = true;
			radiationBlurPlane.material = radiationBlurMaterial;
		}
	}
}
